class NSM_Console
  NSMVERSION = "0.3"
  
  def initialize(args)
    ## Print the lobster
    print_banner()
    puts "NSM Console version #{NSMVERSION}"
    print "\n"

    ## Set {$PCAP_FILE} if passed in as an argument
    CommandManager.execute("file",args[0]) if args.length > 0

    ## Load modules
    load_modules($moduledir)
    load_categories($moduledir)

    ## Initialize logging
    logfilename = "logs/nsm-log."
    logfilename.concat(Time.now.year.to_s)
    logfilename.concat(Time.now.month.to_s)
    logfilename.concat(Time.now.day.to_s)
    logfilename.concat(".log")
    
    start_logging(logfilename)

    puts "Default ${OUTPUT_DIR} is '#{$outputdir}'"
    puts "Default ${MODULE_DIR} is '#{$moduledir}'"

    cmd = ""

    print "\n"    
  end

  def start_logging(logfilename)
    puts "Logging to #{logfilename}\n\n"
    Logger.new(logfilename)
    ## Write initial entry
    Logger.write("-" * 80)
    Logger.write("\n")
    Logger.write("Log for nsm-console begun at #{Time.now.year}-#{Time.now.month}-#{Time.now.day} #{Time.now.hour}:#{Time.now.min}:#{Time.now.sec}\n")
    Logger.write("-" * 80)
    Logger.write("\n")
  end

  def start_shell
    ## Command loop until quit
    while (true)

      ## New method uses readline so we can use tab-completion
      cmd = readline("nsm> ", TRUE)

      ## Split the command into function and arguments
      unless cmd.nil?
        cmd.gsub!(/(\w)\s([\s\S]*)/) { $1 }
        cmd.chomp!
        args = $2
        args.strip! unless args.nil?

        begin
          Logger.write("[nsmcmd] #{cmd} #{args}\n")
          History.write("#{cmd} #{args}")
          CommandManager.execute(cmd,args) unless cmd.length < 1
        rescue LocalJumpError
          ## Ignore LocalJumpErrors, it just means we returned
        rescue NoMethodError
          puts "Command '#{cmd}' unrecognized. Try 'help' for a list of commands."
        rescue
          puts "Error encountered: #{$!}"
        end
      end

    end
  end

  def run
    puts "=-" * 35
    puts "Welcome to NSM Console, type 'help' to see available commands"
    puts "=-" * 35
    puts "Note: All modules are DISABLED by default, use 'list' to list available"
    puts "modules and 'toggle <module>' to disable/enable a module.\n\n"

    start_shell()
  end
end